from dataclasses import dataclass
from typing import (
    Any,
    Dict,
    List,
)

from horsebox.cli.render import render_error

from .filter_type import FilterType
from .tokenizer_type import TokenizerType

CustomAnalyzerArgs = Dict[str, Any]


@dataclass
class CustomAnalyzerDef:
    """Definition of a custom analyzer."""

    tokenizer: Dict[TokenizerType, CustomAnalyzerArgs]
    filters: List[Dict[FilterType, CustomAnalyzerArgs]]

    def __post_init__(self) -> None:  # noqa: D105
        if not isinstance(self.tokenizer, dict):
            render_error(f'Incorrect tokenizer: {self.tokenizer}')
        if len(self.tokenizer) != 1:
            render_error(f'Only one tokenizer is allowed: {",".join(list(self.tokenizer.keys()))}')

        # Convert enumerations serialized as a string back to the enumeration type
        tokenizer_type, tokenizer_params = self.tokenizer.popitem()
        self.tokenizer[TokenizerType(tokenizer_type)] = tokenizer_params

        filters: List[Dict[FilterType, CustomAnalyzerArgs]] = []
        for filter in self.filters:
            filter_type, filter_params = filter.popitem()
            filters.append({FilterType(filter_type): filter_params})
        self.filters = filters



import os
import urllib.request
from typing import (
    Any,
    Generator,
    Iterable,
    List,
)

from trafilatura import extract

from horsebox.cli import FILENAME_PREFIX
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector
from horsebox.utils.ipv6 import ipv6_disabled


class CollectorHtml(Collector):
    """
    HTML Collector Class.

    Used to collect the content of an HTML page.
    """

    pages: List[str]

    def __init__(  # noqa: D107
        self,
        pages: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        self.pages = pages

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorHtml(
            kwargs.pop('root_path'),
            **kwargs,
        )

    def collect(self) -> Iterable[TDocument]:
        """
        Collect the data to index.

        Returns:
            Iterable[TDocument]: The collected documents.
        """
        for page in self.pages:
            yield from self.parse('', page)

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its content.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        if self.dry_run:
            yield prepare_doc(
                path=file_path[1:] if file_path.startswith(FILENAME_PREFIX) else file_path,
            )
            return

        if file_path.startswith(FILENAME_PREFIX):
            with open(file_path[1:], 'r') as file:
                content = file.read()
        else:
            with ipv6_disabled():
                with urllib.request.urlopen(file_path) as response:
                    content = response.read()

        if extracted := extract(
            content,
            include_tables=False,
            include_comments=False,
        ):
            yield prepare_doc(
                **{
                    'name': os.path.basename(file_path),
                    'path': file_path,
                    'content': extracted,
                }
            )



import json
from typing import (
    Any,
    Dict,
    List,
    Optional,
)

import click
import tantivy

from horsebox.cli.param_parser import (
    parse_params,
    parse_params_group,
)
from horsebox.cli.render import render_error
from horsebox.indexer.analyzer import (
    CustomAnalyzerDef,
    FilterType,
    TokenizerType,
)
from horsebox.indexer.analyzer.custom_analyzer_def import CustomAnalyzerArgs
from horsebox.utils.strings import ellipsize


def get_custom_analyzer_def(
    tokenizer_type: TokenizerType,
    tokenizer_params: Optional[str],
    filter_types: List[FilterType],
    filter_params: Optional[str],
) -> CustomAnalyzerDef:
    """
    Get a custom analyzer definition form raw parameters.

    Args:
        tokenizer_type (TokenizerType): The tokenizer to use.
        tokenizer_params (Optional[str]): The parameters of the tokenizer.
        filter_types (List[FilterType]): The filters to use.
        filter_params (Optional[str]): The parameters of the filters.
    """
    filters: List[Dict[FilterType, CustomAnalyzerArgs]] = []

    filter_params_group = parse_params_group(filter_params, len(filter_types))
    for filter_type, filter_params in zip(filter_types, filter_params_group):
        filters.append({FilterType(filter_type): parse_params(filter_params)})

    return CustomAnalyzerDef(
        tokenizer={
            tokenizer_type: parse_params(
                tokenizer_params,
                is_raw=tokenizer_type in [TokenizerType.REGEX],
            ),
        },
        filters=filters,
    )


def load_custom_analyzer_def(filename: str) -> CustomAnalyzerDef:
    """
    Load the definition of a custom analyzer from a file.

    Args:
        filename (str): The name of the file.
    """
    content: Dict[str, Any]

    try:
        with click.open_file(filename, 'r') as file:
            content = json.load(file)
    except json.decoder.JSONDecodeError as e:
        render_error(f'Invalid custom analyzer JSON file: {filename} - {e}')

    try:
        return CustomAnalyzerDef(**content)
    except Exception as e:
        dump = json.dumps(content)
        render_error(f'Error while parsing custom analyzer definition: {filename} - {e} - {ellipsize(dump, 50)}')
        raise


def get_analyzer(custom_analyzer: CustomAnalyzerDef) -> tantivy.TextAnalyzer:
    """
    Create a text analyzer.

    Attention! The configuration `HB_CUSTOM_STOPWORDS` is ignored.

    Args:
        custom_analyzer (CustomAnalyzer): The custom analyzer definition to use.
    """
    tokenizer_type, tokenizer_params = next(iter(custom_analyzer.tokenizer.items()))
    tokenizer_factory = getattr(tantivy.Tokenizer, tokenizer_type.value)
    t_tokenizer: tantivy.Tokenizer = tokenizer_factory(**tokenizer_params)

    builder = tantivy.TextAnalyzerBuilder(t_tokenizer)

    for filter in custom_analyzer.filters:
        filter_type, filter_param = next(iter(filter.items()))

        filter_factory = getattr(tantivy.Filter, filter_type.value)
        t_filter: tantivy.Filter = filter_factory(**filter_param)
        builder = builder.filter(t_filter)

    return builder.build()



import os
from datetime import datetime
from typing import (
    Any,
    Generator,
    List,
    Optional,
)

import click

from horsebox.cli.config import config
from horsebox.collectors import FILENAME_PIPE
from horsebox.collectors.collector_fs.collector import CollectorFS
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector
from horsebox.utils.normalize import normalize_string


class CollectorFSByContent(CollectorFS):
    """By Content File System Collector Class."""

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        pattern: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(
            root_path,
            pattern,
            **kwargs,
        )

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorFSByContent(
            kwargs.pop('root_path'),
            kwargs.pop('pattern'),
            **kwargs,
        )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its content.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        stats: Optional[os.stat_result] = None
        filename: str

        if file_path == FILENAME_PIPE:
            filename = FILENAME_PIPE
        else:
            stats = os.stat(file_path)
            if config.parser_max_content is not None and stats.st_size > config.parser_max_content:
                return

            filename = file_path[len(root_path) :].strip('/')

        _, ext = os.path.splitext(filename)
        content: List[str] = []

        if not self.dry_run:
            parser_max_line = config.parser_max_line

            with click.open_file(
                file_path,
                'r',
                errors='surrogateescape',
            ) as file:
                # Process file line-by-line to limit memory allocation during string normalization
                for line in file:
                    if parser_max_line is not None and len(line) > parser_max_line:
                        continue

                    line = line.strip()
                    if config.string_normalize:
                        line = normalize_string(line)
                    if not line:
                        continue

                    content.append(line)

        yield prepare_doc(
            name=filename,
            type=ext,
            content=content,
            path=file_path,
            size=stats.st_size if stats else None,
            # Time of most recent content modification expressed in seconds.
            date=datetime.fromtimestamp(stats.st_mtime) if stats else None,
        )



import json
from datetime import datetime
from typing import (
    Any,
    Generator,
    List,
)

from horsebox.formatters import DATE_FORMAT
from horsebox.model import TOutput


def __encoder(value: Any) -> str:
    """
    Return a JSON string representation of a Python object.

    >>> __encoder('lorem')
    'lorem'

    >>> __encoder(None)
    ''

    >>> __encoder(datetime(2025, 4, 13, 12, 34, 56))
    '2025-04-13 12:34:56'
    """
    if value is None:
        return ''
    elif isinstance(value, datetime):
        return value.strftime(DATE_FORMAT)

    return str(value)


def format_json(output: TOutput) -> Generator[str, Any, None]:
    """
    JSON formatter.

    Args:
        output (TOutput): The content to format.

    Yields:
        Generator[str, Any, None]: The formatted content.
    """
    yield json.dumps(
        output[0] if isinstance(output, List) and len(output) == 1 else output,
        indent=2,
        default=__encoder,
    )



import os
from datetime import datetime
from typing import (
    Any,
    Generator,
    List,
)

from horsebox.collectors.collector_fs.collector import CollectorFS
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector


class CollectorFSByFilename(CollectorFS):
    """By Filename File System Collector Class."""

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        pattern: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(
            root_path,
            pattern,
            **kwargs,
        )

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorFSByFilename(
            kwargs.pop('root_path'),
            kwargs.pop('pattern'),
            **kwargs,
        )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its filename.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        stats: os.stat_result = os.stat(file_path)

        filename = file_path[len(root_path) :].strip('/')
        _, ext = os.path.splitext(filename)

        # No difference between the returned document in real-mode or in dry-mode
        yield prepare_doc(
            name=filename,
            type=ext,
            path=file_path,
            size=stats.st_size,
            # Time of most recent content modification expressed in seconds.
            date=datetime.fromtimestamp(stats.st_mtime),
        )



from enum import Enum


class FilterType(str, Enum):
    """
    Types of Filters.

    See https://docs.rs/tantivy/latest/tantivy/tokenizer/trait.TokenFilter.html.
    """

    ALPHANUM_ONLY = 'alphanum_only'
    """Removes all tokens that contain non ascii alphanumeric characters."""
    ASCII_FOLD = 'ascii_fold'
    """
    Converts alphabetic, numeric, and symbolic Unicode characters which are not in the first 127 ASCII characters
    into their ASCII equivalents, if one exists.
    """
    LOWERCASE = 'lowercase'
    """Lowercase terms."""
    REMOVE_LONG = 'remove_long'
    """
    Removes tokens that are longer than a given number of bytes.

    Args:
        length_limit (int)
    """
    STEMMER = 'stemmer'
    """
    Stemmer token filter.

    Tokens are expected to be lowercased beforehand.

    Args:
        language (str)
    """
    STOPWORD = 'stopword'
    """
    Removes stop words for a given language.

    Args:
        language (str)
    """
    CUSTOM_STOPWORD = 'custom_stopword'
    """
    Removes stop words from a given a list.

    Args:
        stopwords (List[str])
    """
    SPLIT_COMPOUND = 'split_compound'
    """
    Splits compound words into their parts based on a given dictionary.

    Args:
        constituent_words: (List[str])
    """



import json
import os
import shutil
from dataclasses import asdict
from datetime import datetime
from typing import (
    Any,
    Dict,
    Optional,
)

import tantivy

from horsebox import __version__
from horsebox.cli.render import render_error
from horsebox.indexer.build_args import IndexBuildArgs

__METADATA_FILENAME = 'meta.json'
__METADATA_TIMESTAMP = 'timestamp'
__METADATA_BUILD_ARGS = 'build_args'
__METADATA_VERSION = 'version'


def __read_metadata(index: str) -> Dict[str, Any]:
    if not tantivy.Index.exists(index):
        render_error(f'No index was found at {index}')

    with open(os.path.join(index, __METADATA_FILENAME), 'r') as file:
        meta: Dict[str, Any] = json.load(file)

    return meta


def __write_metadata(
    index: str,
    metadata: Dict[str, Any],
) -> None:
    if not tantivy.Index.exists(index):
        render_error(f'No index was found at {index}')

    filename = os.path.join(index, __METADATA_FILENAME)
    # Make a backup copy of the file `meta.json` to recover from potential corruption
    shutil.copyfile(filename, filename + '.bak')

    metadata[__METADATA_VERSION] = __version__

    with open(filename, 'w') as file:
        json.dump(metadata, file)


def get_timestamp(index: str) -> Optional[datetime]:
    """
    Get the date of creation of an index.

    Args:
        index (str): The path of the index.
    """
    meta = __read_metadata(index)
    if timestamp := meta.get(__METADATA_TIMESTAMP):
        return datetime.fromtimestamp(timestamp)

    return None


def set_timestamp(
    index: str,
    timestamp: datetime,
) -> None:
    """
    Set the date of creation of an index.

    Args:
        index (str): The path of the index.
        timestamp (datetime): The date of creation of the index.
    """
    meta = __read_metadata(index)
    meta[__METADATA_TIMESTAMP] = timestamp.timestamp()
    __write_metadata(index, meta)


def get_build_args(index: str) -> Optional[IndexBuildArgs]:
    """
    Get the build arguments of an index.

    Args:
        index (str): The path of the index.
    """
    meta = __read_metadata(index)
    if build_args := meta.get(__METADATA_BUILD_ARGS):
        return IndexBuildArgs(**build_args)

    return None


def set_build_args(
    index: str,
    build_args: IndexBuildArgs,
) -> None:
    """
    Set the build arguments of an index.

    Args:
        index (str): The path of the index.
        build_args (IndexBuildArgs): The arguments used to build the index.
    """
    meta = __read_metadata(index)
    meta[__METADATA_BUILD_ARGS] = asdict(build_args)
    __write_metadata(index, meta)


def set_metadata(
    index: str,
    timestamp: datetime,
    build_args: Optional[IndexBuildArgs],
) -> None:
    """
    Set (atomically) the metadata of an index.

    Args:
        index (str): The path of the index.
        timestamp (datetime): The date of creation of the index.
        build_args (Optional[IndexBuildArgs]): The arguments used to build the index.
    """
    meta = __read_metadata(index)
    meta[__METADATA_TIMESTAMP] = timestamp.timestamp()
    if build_args:
        meta[__METADATA_BUILD_ARGS] = asdict(build_args)
    __write_metadata(index, meta)



import os
from typing import (
    Any,
    Generator,
    Iterable,
    List,
)

import click
import ijson

from horsebox.cli import (
    FILENAME_PREFIX,
    OPTION_COLLECT_AS_JSONL,
)
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector


class CollectorRaw(Collector):
    """
    Raw Collector Class.

    Used to collect ready to index JSON documents.
    """

    root_path: List[str]
    collect_as_jsonl: bool

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        collect_as_jsonl: bool,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        self.root_path = root_path
        self.collect_as_jsonl = collect_as_jsonl

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorRaw(
            kwargs.pop('root_path'),
            kwargs.pop(OPTION_COLLECT_AS_JSONL),
            **kwargs,
        )

    def collect(self) -> Iterable[TDocument]:
        """
        Collect the documents to index.

        Returns:
            Iterable[TDocument]: The collected documents.
        """
        for root_path in self.root_path:
            yield from self.parse(
                '',
                root_path[1:] if root_path.startswith(FILENAME_PREFIX) else root_path,
            )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its content.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        filename = file_path

        if self.dry_run:
            yield prepare_doc(
                path=file_path,
            )
            return

        with click.open_file(filename, 'r') as file:
            if self.__is_jsonl(filename):
                # See https://pypi.org/project/ijson/#options-1
                items = ijson.items(file, '', multiple_values=True)
            else:
                items = ijson.items(file, 'item')

            for item in items:
                yield prepare_doc(**item)

    def __is_jsonl(
        self,
        filename: str,
    ) -> bool:
        _, fileext = os.path.splitext(filename)
        return self.collect_as_jsonl or fileext in ['.jsonl', '.ndjson']



from typing import (
    Any,
    Dict,
    Optional,
    Tuple,
)

from horsebox.cli import OPTION_COLLECT_AS_JSONL
from horsebox.collectors import CollectorType


def guess_raw(item: str) -> Optional[Tuple[CollectorType, Dict[str, Any]]]:
    """
    Guess the use of the RAW collector from an item.

    >>> guess_raw('raw.txt') is None
    True

    >>> guess_raw('raw.json')
    (<CollectorType.RAW: 'raw'>, {})

    >>> guess_raw('raw.jsonl')
    (<CollectorType.RAW: 'raw'>, {'collect_as_jsonl': True})

    Args:
        item (str): The item to check.

    Returns:
        Optional[Tuple[CollectorType, Dict[str, Any]]]:
        - The type of the collector.
        - Some extra arguments to use with the collector.
    """
    if item.endswith('.json'):
        return (CollectorType.RAW, {})
    elif item.endswith('.jsonl'):
        return (CollectorType.RAW, {OPTION_COLLECT_AS_JSONL: True})

    return None



from abc import (
    ABC,
    abstractmethod,
)
from typing import (
    Any,
    Generator,
    Iterable,
)

from horsebox.cli import OPTION_DRY_RUN
from horsebox.model import TDocument


class Collector(ABC):
    """Collector Class."""

    dry_run: bool = False

    def __init__(  # noqa: D107
        self,
        **kwargs: Any,
    ) -> None:
        self.dry_run = kwargs.get(OPTION_DRY_RUN, False)

    @staticmethod
    @abstractmethod
    def create_instance(**kwargs: Any) -> 'Collector':
        """Create an instance of the collector."""
        ...

    @abstractmethod
    def collect(self) -> Iterable[TDocument]:
        """
        Collect the documents to index.

        Returns:
            Iterable[TDocument]: The collected documents.
        """
        ...

    @abstractmethod
    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a container for indexing.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index.
        """
        ...



from typing import (
    Any,
    List,
    Mapping,
    Tuple,
)

import click


class CombinedOption(click.Option):
    """Combined Options Support Class."""

    def __init__(  # noqa: D107
        self,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        self.required_if = kwargs.pop('required_if')
        self.ignore_if = kwargs.pop('ignore_if', None)
        self.fallback_value = kwargs.pop('fallback_value', None)

        super().__init__(*args, **kwargs)

    def handle_parse_result(  # noqa: D102
        self,
        ctx: click.Context,
        opts: Mapping[str, Any],
        args: List[str],
    ) -> Tuple[Any, List[str]]:
        if (
            # The option can't be ignored due to the presence of another option
            self.ignore_if not in opts
            # The associated option is not provided
            and self.required_if not in opts
        ):
            raise click.UsageError(f'Option {self.required_if} is required with {self.name}')

        if self.name not in opts and self.fallback_value:
            # `opts` is not mutable, use the default value
            self.default = self.fallback_value

        return super().handle_parse_result(ctx, opts, args)  # type: ignore[no-any-return]



from collections import OrderedDict

from horsebox.cli.render import (
    Format,
    render,
)
from horsebox.indexer.schema import SCHEMA_FIELDS


def schema(format: Format) -> None:
    """
    Show the schema of the index.

    Args:
        format (Format): The rendering format to use.
    """
    output = OrderedDict()

    for field in SCHEMA_FIELDS:
        output[field.name] = {
            'type': field.type.value,
            'description': field.description,
        }

    render(output, format)



import logging
import os
from datetime import datetime
from typing import (
    Any,
    Generator,
    List,
)

from pdfminer.high_level import extract_pages
from pdfminer.layout import LTTextContainer

from horsebox.cli.config import config
from horsebox.collectors.collector_fs.collector import CollectorFS
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector
from horsebox.utils.normalize import normalize_string

# Disable logging messages of PDFMiner
logging.getLogger('pdfminer').setLevel(logging.ERROR)


class CollectorPdf(CollectorFS):
    """
    PDF Collector Class.

    Used to collect the content of a PDF document.
    """

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        pattern: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(
            root_path,
            pattern,
            **kwargs,
        )

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorPdf(
            kwargs.pop('root_path'),
            kwargs.pop('pattern'),
            **kwargs,
        )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a PDF document for indexing by its content.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        stats: os.stat_result = os.stat(file_path)
        if config.parser_max_content is not None and stats.st_size > config.parser_max_content:
            return

        filename = os.path.basename(file_path)
        _, ext = os.path.splitext(filename)
        # Time of most recent content modification expressed in seconds.
        file_date = datetime.fromtimestamp(stats.st_mtime)

        if self.dry_run:
            yield prepare_doc(
                name=filename,
                type=ext,
                path=file_path,
                size=stats.st_size,
                # Time of most recent content modification expressed in seconds.
                date=datetime.fromtimestamp(stats.st_mtime),
            )
            return

        for pos, page in enumerate(extract_pages(file_path), start=1):
            content: List[str] = []
            # Process file block-by-block to limit memory allocation during string normalization
            for element in page:
                if isinstance(element, LTTextContainer):
                    text = element.get_text().strip()

                    if config.string_normalize:
                        text = normalize_string(text)
                    if text:
                        content.append(text)

            if not content:
                continue

            yield prepare_doc(
                **{
                    'name': filename,
                    'type': ext,
                    'path': f'{file_path}#L{pos}',
                    'content': content,
                    'size': stats.st_size,
                    'date': file_date,
                }
            )



from enum import Enum


class TokenizerType(str, Enum):
    """
    Types of Tokenizers.

    See https://docs.rs/tantivy/latest/tantivy/tokenizer/trait.Tokenizer.html.
    """

    RAW = 'raw'
    """For each value of the field, emit a single unprocessed token."""
    SIMPLE = 'simple'
    """Tokenize the text by splitting on whitespaces and punctuation."""
    WHITESPACE = 'whitespace'
    """Tokenize the text by splitting on whitespaces."""
    FACET = 'facet'
    """Process a Facet binary representation and emits a token for all of its parent."""
    REGEX = 'regex'
    """
    Tokenize the text by using a regex pattern to split.
    
    Args:
        pattern (str)
    """
    NGRAM = 'ngram'
    """
    Tokenize the text by splitting words into n-grams of the given size(s).

    Args:
        min_gram (int) = 2
        max_gram (int) = 3
        prefix_only (bool) = False
    """



from dataclasses import dataclass
from typing import (
    List,
    Optional,
)

from horsebox.collectors import CollectorType
from horsebox.indexer.analyzer import CustomAnalyzerDef


@dataclass
class IndexBuildArgs:
    """Arguments used to build an index."""

    source: List[str]
    """Locations from which to start indexing."""
    pattern: List[str]
    """The containers to index."""
    collector_type: CollectorType
    """The collector to use."""
    collect_as_jsonl: bool
    """Whether the JSON documents should be collected as JSON Lines or not."""
    custom_analyzer: Optional[CustomAnalyzerDef] = None
    """Custom analyzer."""

    def __post_init__(self) -> None:  # noqa: D105
        # Convert enumerations serialized as a string back to the enumeration type
        self.collector_type = CollectorType(self.collector_type)

        if isinstance(self.custom_analyzer, dict):
            # Create from JSON deserialization
            self.custom_analyzer = CustomAnalyzerDef(**self.custom_analyzer)



from typing import (
    Any,
    Dict,
    List,
    Tuple,
)
from urllib.parse import urlparse

from horsebox.cli import PATTERN_ANY
from horsebox.cli.render import render_error
from horsebox.collectors import CollectorType
from horsebox.collectors.guessers import (
    guess_html,
    guess_pdf,
    guess_raw,
    guess_rss,
)


def guess_collector(
    collector_type: CollectorType,
    source: List[str],
    pattern: List[str],
) -> Tuple[CollectorType, Dict[str, Any]]:
    """
    Guess the collector to use.

    >>> guess_collector(CollectorType.FILECONTENT, [], [])
    (<CollectorType.FILECONTENT: 'filecontent'>, {})

    >>> guess_collector(CollectorType.FILECONTENT, [], ['*'])
    (<CollectorType.FILECONTENT: 'filecontent'>, {})

    >>> guess_collector(CollectorType.FILECONTENT, [], ['*.md'])
    (<CollectorType.FILECONTENT: 'filecontent'>, {})

    >>> guess_collector(CollectorType.GUESS, [], [])
    (<CollectorType.GUESS: 'guess'>, {})

    >>> guess_collector(CollectorType.GUESS, ['raw.json'], [])
    (<CollectorType.RAW: 'raw'>, {})

    >>> guess_collector(CollectorType.GUESS, ['raw.jsonl'], [])
    (<CollectorType.RAW: 'raw'>, {'collect_as_jsonl': True})

    >>> guess_collector(CollectorType.GUESS, [], ['*.txt'])
    (<CollectorType.FILECONTENT: 'filecontent'>, {})

    >>> guess_collector(CollectorType.GUESS, [], ['*.pdf'])
    (<CollectorType.PDF: 'pdf'>, {})

    >>> guess_collector(CollectorType.GUESS, ['https://planetpython.org/rss20.xml'], [])
    (<CollectorType.RSS: 'rss'>, {})

    >>> guess_collector(CollectorType.GUESS, ['@planetpython.xml'], [])
    (<CollectorType.RSS: 'rss'>, {})

    >>> guess_collector(CollectorType.GUESS, ['@planetpython.atom'], [])
    (<CollectorType.RSS: 'rss'>, {})

    >>> guess_collector(CollectorType.GUESS, ['https://en.wikipedia.org/wiki/Python_(programming_language)'], [])
    (<CollectorType.HTML: 'html'>, {})

    >>> guess_collector(CollectorType.GUESS, ['@Python_(programming_language).html'], [])
    (<CollectorType.HTML: 'html'>, {})

    >>> guess_collector(CollectorType.GUESS, ['file.pdf'], [])
    (<CollectorType.PDF: 'pdf'>, {})

    Args:
        collector_type (CollectorType): The provided type of the collector.
        source (List[str]): The provided locations from which to start indexing.
        pattern (List[str]): The provided containers to index.

    Returns:
        Tuple[CollectorType, Dict[str, Any]]:
        - The type of the collector.
        - Some extra arguments to use with the collector.
    """
    if collector_type != CollectorType.GUESS:
        return (collector_type, {})

    if pattern := [p for p in pattern if p != PATTERN_ANY]:
        if guess := guess_pdf(pattern[0]):
            return guess

        # Other patterns are only supported by File System Collectors.
        # Use the File System Collector by default.
        return (CollectorType.FILECONTENT, {})

    for s in source:
        parsed = urlparse(s)
        if parsed.scheme in ['http', 'https']:
            # Online source
            if guess := guess_rss(parsed.path):
                return guess
            else:
                # If it's not an RSS feed, then it has to be an HTML page
                return (CollectorType.HTML, {})
        elif parsed.scheme:
            render_error(f'Unsupported scheme {parsed.scheme}')
        elif (
            (guess := guess_html(parsed.path))
            or (guess := guess_rss(parsed.path))
            or (guess := guess_raw(parsed.path))
            or (guess := guess_pdf(parsed.path))
        ):
            # Offline source
            return guess

        # Extra detection of files provided with the option "--from" may lead to ambiguous results:
        # files with the extension .txt or .md can be detected with confidence, but what about other extensions?
        # In such cases, it is better to explicitly provide the collector to use.

    return (collector_type, {})



from typing import (
    Any,
    Dict,
    List,
    Tuple,
    Type,
)

from horsebox.cli.render import render_error
from horsebox.collectors import CollectorType
from horsebox.collectors.collector_fs import (
    CollectorFSByContent,
    CollectorFSByFilename,
    CollectorFSByLine,
)
from horsebox.collectors.collector_guess import guess_collector
from horsebox.collectors.collector_html import CollectorHtml
from horsebox.collectors.collector_pdf import CollectorPdf
from horsebox.collectors.collector_raw import CollectorRaw
from horsebox.collectors.collector_rss import CollectorRSS
from horsebox.model.collector import Collector

__COLLECTORS: Dict[CollectorType, Type[Collector]] = {
    CollectorType.FILENAME: CollectorFSByFilename,
    CollectorType.FILECONTENT: CollectorFSByContent,
    CollectorType.FILELINE: CollectorFSByLine,
    CollectorType.RSS: CollectorRSS,
    CollectorType.RAW: CollectorRaw,
    CollectorType.HTML: CollectorHtml,
    CollectorType.PDF: CollectorPdf,
}


def get_collector(
    collector_type: CollectorType,
    source: List[str],
    pattern: List[str],
) -> Tuple[Type[Collector], Dict[str, Any]]:
    """
    Get a collector factory.

    Args:
        collector_type (CollectorType): The type of the collector.
        source (List[str]): Locations from which to start indexing.
        pattern (List[str]): The containers to index.

    Returns:
        Tuple[Type[Collector], Dict[str, Any]]:
            - The type of the collector.
            - Some extra arguments to use with the collector.
    """
    collector_type, extra_args = guess_collector(
        collector_type,
        source,
        pattern,
    )
    collector = __COLLECTORS.get(collector_type)
    if not collector:
        render_error('No collector found')

    return (collector, extra_args)



import os
from typing import (
    List,
    Set,
    Tuple,
)

from horsebox.cli import PATTERN_ANY


def explode_pattern(
    source: List[str],
    pattern: List[str],
) -> Tuple[List[str], List[str]]:
    """
    Convert locations and containers description to canonical ones.

    >>> explode_pattern([], [])
    ([], [])

    >>> explode_pattern(['*.txt'], ['*'])
    (['.'], ['*.txt'])

    >>> explode_pattern(['this/folder'], ['*'])
    (['this/folder'], ['*'])

    >>> explode_pattern(['this/folder'], ['*.txt'])
    (['this/folder'], ['*.txt'])

    >>> explode_pattern(['this/folder/*.txt'], ['*'])
    (['this/folder/'], ['*.txt'])

    >>> explode_pattern(['this/folder/*.txt'], ['*.pdf'])
    (['this/folder/'], ['*.pdf', '*.txt'])

    >>> explode_pattern(['this/folder*/*.txt'], ['*.pdf'])
    (['this/'], ['*.pdf', 'folder*/*.txt'])

    >>> explode_pattern(['this/folder*/file.txt'], ['*'])
    (['this/'], ['folder*/file.txt'])

    Args:
        source (List[str]): Locations from which to start indexing.
        pattern (List[str]): The containers to index.

    Returns:
        Tuple[List[str], List[str]]: The canonicalized (locations, containers).
    """
    source2: List[str] = []
    pattern2: Set[str] = set(pattern)

    for s in source:
        if PATTERN_ANY in s:
            # Only the magic '*' is supported
            magic_pos = s.find(PATTERN_ANY)
            if sep_last_pos := s.rfind(os.sep, None, magic_pos) + 1:
                root_path = s[:sep_last_pos]
                p = s[sep_last_pos:]
            else:
                root_path = '.'
                p = s

            source2.append(root_path)
            pattern2.add(p)
        else:
            source2.append(s)

    if len(pattern2) > 1:
        # The joker pattern is not required if there are more specific patterns
        pattern2.discard(PATTERN_ANY)

    # HACK Return a sorted list of patterns to allow repeatable tests
    return (source2, sorted(pattern2))



import os
from dataclasses import dataclass
from datetime import timedelta
from typing import (
    Dict,
    Optional,
    Union,
    cast,
)

_CONFIG_INDEX_BATCH_SIZE = 'HB_INDEX_BATCH_SIZE'
_CONFIG_HIGHLIGHT_MAX_CHARS = 'HB_HIGHLIGHT_MAX_CHARS'
_CONFIG_PARSER_MAX_LINE = 'HB_PARSER_MAX_LINE'
_CONFIG_PARSER_MAX_CONTENT = 'HB_PARSER_MAX_CONTENT'
_CONFIG_RENDER_MAX_CONTENT = 'HB_RENDER_MAX_CONTENT'
_CONFIG_INDEX_EXPIRATION = 'HB_INDEX_EXPIRATION'
_CONFIG_CUSTOM_STOPWORDS = 'HB_CUSTOM_STOPWORDS'
_CONFIG_STRING_NORMALIZE = 'HB_STRING_NORMALIZE'
_CONFIG_TOP_MIN_CHARS = 'HB_TOP_MIN_CHARS'


@dataclass
class Config:
    """Configuration Details."""

    description: str
    default: Optional[Union[int, str]] = None


CONFIGS: Dict[str, Config] = {
    _CONFIG_INDEX_BATCH_SIZE: Config('Batch size when indexing.', 1_000),
    _CONFIG_HIGHLIGHT_MAX_CHARS: Config('Maximum number of characters to show for highlights.', 200),
    _CONFIG_PARSER_MAX_LINE: Config('Maximum size of a line in a container (unlimited if null).'),
    _CONFIG_PARSER_MAX_CONTENT: Config('Maximum size of a container (unlimited if null).'),
    _CONFIG_RENDER_MAX_CONTENT: Config('Maximum size of a document content to render (unlimited if null).'),
    _CONFIG_INDEX_EXPIRATION: Config('Index freshness threshold (in seconds).', 3_600),
    _CONFIG_CUSTOM_STOPWORDS: Config('Custom list of stop-words (separated by a comma).'),
    _CONFIG_STRING_NORMALIZE: Config('Normalize strings when reading files (0=disabled, other value=enabled).', 1),
    _CONFIG_TOP_MIN_CHARS: Config('Minimum number of characters of a top keyword.', 1),
}


class __Config:
    """Configuration Class."""

    __index_batch_size: int
    __highlight_max_chars: int
    __parser_max_content: Optional[int]
    __parser_max_line: Optional[int]
    __render_max_content: int
    __index_expiration: timedelta
    __custom_stopwords: Optional[str]
    __string_normalize: bool
    __top_min_chars: int

    def __init__(self) -> None:  # noqa: D107
        self.__index_batch_size = cast(int, self.__get_int(_CONFIG_INDEX_BATCH_SIZE))
        self.__highlight_max_chars = cast(int, self.__get_int(_CONFIG_HIGHLIGHT_MAX_CHARS))
        self.__parser_max_content = self.__get_int(_CONFIG_PARSER_MAX_CONTENT)
        self.__parser_max_line = self.__get_int(_CONFIG_PARSER_MAX_LINE)
        self.__render_max_content = cast(int, self.__get_int(_CONFIG_RENDER_MAX_CONTENT))
        self.__index_expiration = timedelta(seconds=cast(int, self.__get_int(_CONFIG_INDEX_EXPIRATION)))
        self.__custom_stopwords = os.environ.get(_CONFIG_CUSTOM_STOPWORDS)
        self.__string_normalize = bool(self.__get_int(_CONFIG_STRING_NORMALIZE))
        self.__top_min_chars = cast(int, self.__get_int(_CONFIG_TOP_MIN_CHARS))

    def __get_int(
        self,
        key: str,
    ) -> Optional[int]:
        default = CONFIGS[key].default
        value = os.environ.get(key, default)
        try:
            return int(value)  # type: ignore
        except (ValueError, TypeError):
            # Silently fail, and return the default value
            return default  # type: ignore

    @property
    def index_batch_size(self) -> int:
        """Batch size when indexing."""
        return self.__index_batch_size

    @property
    def highlight_max_chars(self) -> int:
        """Maximum number of characters to show for highlights."""
        return self.__highlight_max_chars

    @property
    def parser_max_content(self) -> Optional[int]:
        """
        Maximum size of a container.

        Not limited if `None`.
        """
        return self.__parser_max_content

    @property
    def parser_max_line(self) -> Optional[int]:
        """
        Maximum size of a line in a container.

        Not limited if `None`.
        """
        return self.__parser_max_line

    @property
    def render_max_content(self) -> Optional[int]:
        """
        Maximum size of a document content to render.

        Not limited if `None`.
        """
        return self.__render_max_content

    @property
    def index_expiration(self) -> timedelta:
        """
        Index freshness threshold.

        Not limited if `None`.
        """
        return self.__index_expiration

    @property
    def custom_stopwords(self) -> Optional[str]:
        """Custom list of stop-words."""
        return self.__custom_stopwords

    @property
    def string_normalize(self) -> bool:
        """Normalize strings when reading files."""
        return self.__string_normalize

    @property
    def top_min_chars(self) -> int:
        """Minimum number of characters of a top keyword."""
        return self.__top_min_chars


config = __Config()
"""Configuration."""



from typing import (
    Any,
    Callable,
    Dict,
    Generator,
    List,
    Union,
)

TDocument = Dict[str, Any]
"""Document to index."""

TOutput = Union[List[Dict[str, Any]], Dict[str, Any]]
"""Output to render."""
TFormatter = Callable[[TOutput], Generator[str, Any, None]]
"""Formatter user to render an output."""



import html
from typing import (
    Any,
    Dict,
    Generator,
    Iterable,
    List,
    cast,
)

import feedparser

from horsebox.cli import FILENAME_PREFIX
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector
from horsebox.utils.normalize import strip_html_tags

_RSS_CONTENT = ['title', 'summary']


class CollectorRSS(Collector):
    """RSS Feed Collector Class."""

    feeds: List[str]

    def __init__(  # noqa: D107
        self,
        feeds: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        self.feeds = feeds

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorRSS(
            kwargs.pop('root_path'),
            **kwargs,
        )

    def collect(self) -> Iterable[TDocument]:
        """
        Collect the data to index.

        Returns:
            Iterable[TDocument]: The collected documents.
        """
        for feed in self.feeds:
            yield from self.parse(
                '',
                feed[1:] if feed.startswith(FILENAME_PREFIX) else feed,
            )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its content.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per file).
        """
        feed = file_path

        if self.dry_run:
            yield prepare_doc(
                path=file_path,
            )
            return

        parsed = feedparser.parse(feed[1:] if feed.startswith(FILENAME_PREFIX) else feed)
        for item in parsed.entries:
            yield self.__build_doc(item)

    def __build_doc(
        self,
        item: feedparser.FeedParserDict,
    ) -> TDocument:
        """Build a document to be indexed."""
        # If a deterministic `doc_id` is required (for document update/deletion),
        # it could be generated with `abs(hash(kwargs['path']))`

        doc: TDocument = {}
        contents: Dict[str, str] = {}

        for tag in _RSS_CONTENT:
            if value := item.get(tag):
                contents[tag] = html.unescape(strip_html_tags(cast(str, value)))

        if content := '\n'.join(filter(None, [contents.get(x) for x in _RSS_CONTENT])):
            doc['content'] = content
        if title := contents.get('title'):
            doc['name'] = title

        if value := item.get('link'):
            doc['path'] = value

        for tag in ['updated_parsed', 'published_parsed']:
            if value := item.get(tag):
                doc['date'] = value
                break

        return prepare_doc(**doc)



from typing import List

from horsebox.cli import ELLIPSIS


def split_with_escaped(
    text: str,
    sep: str,
) -> List[str]:
    """
    Return a list of the substrings in a string, with support of an escaped separator.

    An escaped separator is enclosed in curly brackets (e.g. {,} for an escaped comma).

    >>> split_with_escaped('', ',')
    ['']

    >>> split_with_escaped('lorem', ',')
    ['lorem']

    >>> split_with_escaped('lorem,ipsum', ',')
    ['lorem', 'ipsum']

    >>> split_with_escaped('lorem,ipsum{,}dolor', ',')
    ['lorem', 'ipsum,dolor']

    >>> split_with_escaped('{,}lorem,ipsum{,}', ',')
    [',lorem', 'ipsum,']

    >>> split_with_escaped('{,}{,},', ',')
    [',,', '']

    Args:
        text (str): the string to split.
        sep (str): The separator used to split the string.
    """
    pivot_chr = chr(26)  # SUB
    escaped = text.replace('{' + sep + '}', pivot_chr)
    return [
        # Rebuild the part with the original (non-escaped) separator
        part.replace(pivot_chr, sep)
        for part
        # Split the escaped string according to the (non-escaped) separators
        in escaped.split(sep)
    ]


def ellipsize(
    text: str,
    size: int,
) -> str:
    """
    Get the content of a string, limited to a maximum size.

    >>> ellipsize('', 0)
    ''

    >>> ellipsize('lorem', 3)
    '...'

    >>> ellipsize('lorem', 5)
    'lorem'

    >>> ellipsize('lorem', 10)
    'lorem'

    >>> ellipsize('lorem ipsum', 10)
    'lorem i...'

    Args:
        text (str): The text to get the content from.
        size (int): The maximum size to return.
    """
    return text if len(text) <= max(size, len(ELLIPSIS)) else text[: size - len(ELLIPSIS)] + ELLIPSIS



import os
from datetime import datetime
from typing import (
    Any,
    Generator,
    List,
    Optional,
)

import click

from horsebox.cli.config import config
from horsebox.collectors import FILENAME_PIPE
from horsebox.collectors.collector_fs.collector import CollectorFS
from horsebox.indexer.factory import prepare_doc
from horsebox.model import TDocument
from horsebox.model.collector import Collector
from horsebox.utils.normalize import normalize_string


class CollectorFSByLine(CollectorFS):
    """By Line File System Collector Class."""

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        pattern: List[str],
        **kwargs: Any,
    ) -> None:
        super().__init__(
            root_path,
            pattern,
            **kwargs,
        )

    @staticmethod
    def create_instance(**kwargs: Any) -> Collector:
        """Create an instance of the collector."""
        return CollectorFSByLine(
            kwargs.pop('root_path'),
            kwargs.pop('pattern'),
            **kwargs,
        )

    def parse(
        self,
        root_path: str,
        file_path: str,
    ) -> Generator[TDocument, Any, None]:
        """
        Parse a file for indexing by its content with line detail.

        Args:
            root_path (str): Base path of the file.
            file_path (str): File to parse.

        Yields:
            Generator[TDocument, Any, None]: The document to index (one document per line).
        """
        stats: Optional[os.stat_result] = None
        filename: str

        if file_path == FILENAME_PIPE:
            filename = FILENAME_PIPE
        else:
            stats = os.stat(file_path)
            filename = file_path[len(root_path) :].strip('/')

        _, ext = os.path.splitext(filename)

        if self.dry_run:
            yield prepare_doc(
                name=filename,
                type=ext,
                path=file_path,
                size=stats.st_size if stats else None,
                # Time of most recent content modification expressed in seconds.
                date=datetime.fromtimestamp(stats.st_mtime) if stats else None,
            )
            return

        parser_max_line = config.parser_max_line

        with click.open_file(
            file_path,
            'r',
            errors='surrogateescape',
        ) as file:
            for pos, line in enumerate(file, start=1):
                if parser_max_line is not None and len(line) > parser_max_line:
                    continue

                line = line.strip()
                if config.string_normalize:
                    line = normalize_string(line)
                if not line:
                    continue

                yield prepare_doc(
                    name=filename,
                    type=ext,
                    content=line,
                    path=f'{file_path}#{pos}',
                    size=stats.st_size if stats else None,
                    # Time of most recent content modification expressed in seconds.
                    date=datetime.fromtimestamp(stats.st_mtime) if stats else None,
                )



import itertools
import os
from glob import iglob
from typing import (
    Any,
    Iterable,
    List,
)

from horsebox.cli import OPTION_DRY_RUN
from horsebox.collectors import FILENAME_PIPE
from horsebox.model import TDocument
from horsebox.model.collector import Collector


class CollectorFS(Collector):
    """File System Collector Class."""

    root_path: List[str]
    pattern: List[str]

    def __init__(  # noqa: D107
        self,
        root_path: List[str],
        pattern: List[str],
        **kwargs: Any,
    ) -> None:
        self.root_path = root_path
        self.pattern = pattern
        self.dry_run = kwargs.get(OPTION_DRY_RUN, False)

    def collect(self) -> Iterable[TDocument]:
        """
        Collect the data to index.

        Returns:
            Iterable[TDocument]: The collected documents.
        """
        # For each file/folder
        for root_path in self.root_path:
            if root_path == FILENAME_PIPE:
                # Collect from piped content
                yield from self.parse(
                    FILENAME_PIPE,
                    FILENAME_PIPE,
                )
            elif os.path.isfile(root_path):
                # Collect this file
                yield from self.parse(
                    os.path.dirname(root_path),
                    root_path,
                )
            else:
                # For each file in the folder
                for filename in itertools.chain.from_iterable(
                    iglob(
                        os.path.join(os.path.expanduser(root_path), f'**/{p}'),
                        recursive=True,
                    )
                    for p in self.pattern
                ):
                    if not os.path.isfile(filename):
                        continue

                    yield from self.parse(root_path, filename)



from typing import (
    Dict,
    List,
    Optional,
)

import tantivy

from horsebox.cli.config import config
from horsebox.cli.param_parser import parse_params
from horsebox.indexer.analyzer import (
    CustomAnalyzerDef,
    FilterType,
    TokenizerType,
)
from horsebox.indexer.analyzer.custom_analyzer_def import CustomAnalyzerArgs
from horsebox.indexer.analyzer.factory import get_analyzer

__analyzer: Optional[tantivy.TextAnalyzer] = None

_STOP_WORDS_EN_COUNT = 174
"""Number of English stop-words - https://snowballstem.org/algorithms/english/stop.txt"""
_STOP_WORDS_FR_COUNT = 144
"""Number of French stop-words - https://snowballstem.org/algorithms/french/stop.txt"""
STOP_WORDS_COUNT = _STOP_WORDS_EN_COUNT + _STOP_WORDS_FR_COUNT
"""Number of stop-words used in top-keywords analyzer."""


def is_stopword(word: str) -> bool:
    """
    Check if a work is a stop-word or not.

    >>> is_stopword('lorem')
    False

    >>> is_stopword('ipsum')
    False

    >>> is_stopword('and')
    True

    >>> is_stopword('is')
    True

    >>> is_stopword('et')
    True

    >>> is_stopword('il')
    True

    Args:
        word (str): The word to check.
    """
    global __analyzer

    if not __analyzer:
        filters: List[Dict[FilterType, CustomAnalyzerArgs]] = [
            {FilterType.ALPHANUM_ONLY: {}},
            {FilterType.REMOVE_LONG: {'length_limit': 40}},
            {FilterType.LOWERCASE: {}},
            {FilterType.STOPWORD: {'language': 'english'}},
            {FilterType.STOPWORD: {'language': 'french'}},
        ]
        if config.custom_stopwords:
            filters.append(
                {
                    FilterType.CUSTOM_STOPWORD: parse_params(f'stopwords=[{config.custom_stopwords}]'),
                }
            )

        __analyzer = get_analyzer(
            CustomAnalyzerDef(
                tokenizer={
                    TokenizerType.RAW: {},
                },
                filters=filters,
            ),
        )

    analyzed: List[str] = __analyzer.analyze(word)

    return len(analyzed) == 0  # The list will be empty for a stop-word



from typing import (
    List,
    Optional,
    Union,
)

from horsebox.cli import (
    OPTION_COLLECT_AS_JSONL,
    OPTION_DRY_RUN,
)
from horsebox.cli.render import Format
from horsebox.collectors import CollectorType
from horsebox.collectors.factory import get_collector
from horsebox.collectors.pattern import explode_pattern
from horsebox.commands.inspect import inspect
from horsebox.indexer.analyzer import CustomAnalyzerDef
from horsebox.indexer.analyzer.factory import load_custom_analyzer_def
from horsebox.indexer.build_args import IndexBuildArgs
from horsebox.indexer.index import feed_index


def build_index(
    source: List[str],
    pattern: List[str],
    index: str,
    collector_type: CollectorType,
    collect_as_jsonl: bool,
    dry_run: bool,
    analyzer: Optional[Union[str, CustomAnalyzerDef]],
    format: Format,
) -> None:
    """
    Build a persistent index.

    Args:
        source (List[str]): Locations from which to start indexing.
        pattern (List[str]): The containers to index.
        index (str): The location where to persist the index.
        collector_type (CollectorType): The collector to use.
        collect_as_jsonl (bool): Whether the JSON documents should be collected as JSON Lines or not.
        dry_run (bool): Whether the build of the index should be simulated or done.
        analyzer (Optional[Union[str, CustomAnalyzerDef]]):
            The file containing the definition of the custom analyzer, or the custom analyzer definition.
        format (Format): The rendering format to use.
    """
    source, pattern = explode_pattern(source, pattern)
    collector, extra_args = get_collector(
        collector_type,
        source,
        pattern,
    )

    custom_analyzer: Optional[CustomAnalyzerDef] = None
    if isinstance(analyzer, CustomAnalyzerDef):
        custom_analyzer = analyzer
    elif isinstance(analyzer, str):
        custom_analyzer = load_custom_analyzer_def(analyzer)

    feed_index(
        collector.create_instance(
            root_path=source,
            pattern=pattern,
            **(
                {
                    OPTION_COLLECT_AS_JSONL: collect_as_jsonl,
                    OPTION_DRY_RUN: dry_run,
                }
                | extra_args
            ),
        ),
        index,
        IndexBuildArgs(
            source=source,
            pattern=pattern,
            collector_type=collector_type,
            collect_as_jsonl=collect_as_jsonl,
            custom_analyzer=custom_analyzer,
        ),
        format,
    )

    inspect(index, format)



from .analyze import analyze  # noqa: F401
from .build import build_index  # noqa: F401
from .config import config  # noqa: F401
from .inspect import inspect  # noqa: F401
from .refresh import refresh  # noqa: F401
from .schema import schema  # noqa: F401
from .search import search  # noqa: F401
